<?php

/**
 * Plugin name: Basic Pages
 * Description: Creates pages for your website
 * 
 * 
 **/

set_value([

	'admin_route'	=>'admin',
	'plugin_route'	=>'pages',
	'tables'		=> [
		'pages_table' => 'pages',
	],

]);

/** check if all tables exist **/
$db = new \Core\Database;
$tables = get_value()['tables'];

if(!$db->table_exists($tables)){
	dd("Missing database tables in ".plugin_id() ." plugin: ". implode(",", $tables));
	die;
}

/** set user permissions for this plugin **/
add_filter('permissions',function($permissions){

	$permissions[] = 'view_pages';
	$permissions[] = 'add_page';
	$permissions[] = 'edit_page';
	$permissions[] = 'delete_page';

	return $permissions;
});


/** run this after a form submit **/
add_action('controller',function(){

	$req = new \Core\Request;
	$vars = get_value();
	
	$admin_route = $vars['admin_route'];
	$plugin_route = $vars['plugin_route'];

	if(page() == $vars['admin_route'] && URL(1) == $vars['plugin_route'] && $req->posted())
	{
		$ses = new \Core\Session;
		$page = new \BasicPages\Page;

		$id = URL(3) ?? null;
		if($id)
			$row = $page->first(['id'=>$id]);

		if(URL(2) == 'add'){
			require plugin_path('controllers/add-controller.php');
		}else
		if(URL(2) == 'edit'){
			
			require plugin_path('controllers/edit-controller.php');
		}else
		if(URL(2) == 'delete'){

			require plugin_path('controllers/delete-controller.php');
		}

	}
});


/** displays the view file **/
add_action('view2',function(){

	$vars = get_value();

	if(page() == $vars['admin_route'] && URL(1) == $vars['plugin_route'])
		require plugin_path('views/admin/list.php');
});


/** displays the view file **/
add_action('basic-admin_main_content',function(){
	
	$ses = new \Core\Session;
	$vars = get_value();

	$admin_route = $vars['admin_route'];
	$plugin_route = $vars['plugin_route'];
	
	$errors = $vars['errors'] ?? [];

	$page = new \BasicPages\Page;

	if(page() == $vars['admin_route'] && URL(1) == $vars['plugin_route']){

		$id = URL(3) ?? null;
		if($id){
			$page::$query_id = 'get-pages';
			$row = $page->first(['id'=>$id]);
		}

		if(URL(2) == 'add'){

			require plugin_path('views/admin/add.php');
		}else
		if(URL(2) == 'edit'){
			
			require plugin_path('views/admin/edit.php');
		}else
		if(URL(2) == 'delete'){

			require plugin_path('views/admin/delete.php');
		}else
		if(URL(2) == 'view'){

			require plugin_path('views/admin/view.php');
		}else
		{
			$limit = 30;
			$pager = new \Core\Pager($limit);
			$offset = $pager->offset;

			$page->limit = $limit;
			$page->offset = $offset;
			$page::$query_id = 'get-pages';
			
			if(!empty($_GET['find']))
			{
				$find = '%' . trim($_GET['find']) . '%';
				$query = "select * from users where (first_name like :find || last_name like :find) limit $limit offset $offset";
				$rows = $page->query($query,['find'=>$find]);
			}else{
				$rows = $page->getAll();
			}

			require plugin_path('views/admin/list.php');
		}

	}
});


/** add to amin links **/
add_filter('basic-admin_before_admin_links',function($links){

	if(user_can('view_pages'))
	{
		$vars = get_value();

		$obj = (object)[];
		$obj->title = 'Pages';
		$obj->link = ROOT . '/'.$vars['admin_route'].'/'.$vars['plugin_route'];
		$obj->icon = 'fa-solid fa-file-lines';
		$obj->parent = 0;
		$links[] = $obj;
	}

	return $links;
});


/** for manipulating data after a query operation **/
add_filter('after_query',function($data){

	
	if(empty($data['result']))
		return $data;

	foreach ($data['result'] as $key => $row) {
		


	}

	return $data;
});


